<?php
/**
 * Zebrum Lite
 *
 * 
 *
 *      ,    
 *    LICENSE.txt.
 *           URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: zparser.php 658 2009-04-28 07:07:48Z lks $
 */

/** zsource_txt */
require_once 'zengine/classes/zsource/txt.php';

/** zutils */
require_once 'zengine/classes/zutils.php';

/**
 *       
 *
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zparser
{
    /**
     *      
     *
     * @param string $charset ,   
     */
    public function setDefaultCharset ($charset) {
    	if (!empty($charset)) {
    	    $this->charset = $charset;
    	}
    }

    /**
     *    
     *
     * @param string $charset 
     */
    public function setDefaultFilter ($filter) {
        if (!empty($filter)) {
            $this->filter = $filter;
        }
    }

    /**
     *   
     *
     * @param boolean $deleteOld
     */
    public function deleteOldContent ($deleteOld) {
        $this->deleteOld = (bool) $deleteOld;
    }

    /**
     *      
     *
     * @param boolean $force
     */
    public function forceIndex ($force) {
    	$this->forceIndex = (bool) $force;
    }

    /**
     *        
     *
     * @return boolean
     */
    public function parse ($filename)
    {
        $this->init();
        $this->position = 1;
        $f = fopen($filename, 'r', false);
        if (!$f) {
            return false;
        }
        while (!feof($f)) {
            $this->convert(fread($f, 16*1024), feof($f));
        }
        fclose($f);

        if ($this->deleteOld) {
            /**
             *    
             */
            foreach ($this->oldPages as $pagepath) {
                if (zengine::zsource()->delete($pagepath)) {
                    $this->log('  : ' . $pagepath);
                } else {
                    $this->log('    : ' . $pagepath);
                }
            }
        }

        $this->log(' .');
        return true;
    }

    /**
     *     
     *
     * @var array
     */
    public function getLog () {
        return $this->logs;
    }

    /**
     *      
     */
    private function init () {
        $this->zsource_txt = new zsource_txt();
        $this->buffer = '';
        $this->nextPageLevel = 0;
        $this->levels = array();
        /**
         *     ,     
         * 
         */
        $this->oldPages = array();
        if ($this->deleteOld) {
            $this->oldPages = array_reverse(zengine::zsource()->getAllPages(), true);
        }
    }

    /**
     *   
     *
     * @param string $data  
     * @param boolean $last    
     */
    private function convert ($data, $last) {
    	$this->buffer .= $data;
    	$matches = array();
    	while (preg_match("%^(##+)%m", $this->buffer, $matches, PREG_OFFSET_CAPTURE)) {
    	   $this->savePage(substr($this->buffer, 0, $matches[0][1]));
           $this->nextPageLevel = strlen($matches[0][0]) - 2;
    	   $this->buffer = '@@title=' . substr($this->buffer, $matches[0][1] + strlen($matches[0][0]));
    	}

    	if ($last) {
    	    $this->savePage($this->buffer);
    	}
    }

    /**
     *        
     *
     * @param string $page  
     */
    private function savePage ($page) {
        if (empty($page)) {
            return;
        }

        /**
         *      
         */
    	$page = $this->zsource_txt->parse(preg_replace("%\r?\n%", "\n", $page));
    	if (empty($page['sort'])) {
    	    $page['sort'] = (int) ++$this->position;
    	}
    	if (empty($page['title'])) {
    	    $this->log('   : ');
    	    return;
    	}
    	if (empty($page['charset'])) {
    	    $page['charset'] = $this->charset;
    	}
    	if (empty($page['filter'])) {
    	    $page['filter'] = $this->filter;
    	}
    	if (empty($page['type'])) {
    	    $page['type'] = 'page';
    	}
    	/**
    	 *   
    	 */
    	$pagename = $page['title'];
    	if (!empty($page['file'])) {
    	    $pagename = $page['file'];
    	    unset($page['file']);
    	}
    	if (strcasecmp($page['charset'], 'utf-8') != 0) {
    	    $pagename = zutils::filename(iconv($page['charset'], 'utf-8', $pagename));
    	} else {
    	    $pagename = zutils::filename($pagename);
    	}
        if (empty($pagename)) {
            $this->log('      .  "untitled".');
            $pagename = 'untitled';
        }
    	if ($this->nextPageLevel == 0 && ($this->forceIndex || !zengine::zsource()->exists('/index'))) {
    	    $pagename = 'index';
    	    $this->forceIndex = false;
    	}

    	/**
    	 *      
    	 */
    	$this->levels[$this->nextPageLevel] = $pagename;
    	$directory = implode('/', array_slice($this->levels, 0, $this->nextPageLevel));
    	assert(strpos($directory, '//') === false);
    	$pagepath = '/' . (empty($directory) ? $pagename : $directory . '/' . $pagename);
        $this->log(' : ' . $pagepath);

    	/**
    	 *     ,     
    	 */
    	if (isset($this->oldPages[$pagepath])) {
    	    unset($this->oldPages[$pagepath]);
    	}

    	/**
    	 *  
    	 */
    	zengine::zsource()->save($pagepath, $page);
    }

    /**
     *    
     *
     * @param string $message
     */
    private function log ($message) {
    	$this->logs[] = $message;
    }

    /**
     *    .     
     *
     * @var array
     */
    private $levels = array();

    /**
     * ,   
     *
     * @var string
     */
    private $charset = 'utf-8';

    /**
     * 
     *
     * @var array
     */
    private $logs = array();

    /**
     *        
     *
     * @var string
     */
    private $buffer;

    /**
     *  ,       
     *   
     *
     * @var int
     */
    private $nextPageLevel;

    /**
     *  ,       
     *
     * @var zsource_txt
     */
    private $zsource_txt;

    /**
     *     
     *
     * @var boolean
     */
    private $forceIndex = false;

    /**
     *    
     *
     * @var boolean
     */
    private $deleteOld = true;

    /**
     *   ,    
     *
     * @var array
     */
    private $oldPages = array();

    /**
     *   
     *
     * @var string
     */
    private $filter = 'text2html';

    /**
     *   
     *
     * @var int
     */
    private $position;
}
