<?php
/**
 * Zebrum Lite
 *
 * 
 *
 *      ,    
 *    LICENSE.txt.
 *           URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: defaults.php 669 2009-04-28 18:04:20Z lks $
 */


/**
 * @package    zengine
 * @subpackage widgets
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zwidget_pages extends zwidget
{
    /**
     *   
     */
    public function displayContent ()
    {
        $this->nofollow = '';
        if (zengine::config()->get('menu.nofollow')) {
            $this->nofollow = ' rel="nofollow"';
        }

        $this->zpage = zengine::ztemplate()->zpage;
        $this->pagepath = $this->zpage->getPagepath();

        $menu = zengine::zsource()->getMenu()->find($this->getParam('childs_of'));
        if (!$menu) {
            return;
        }

        echo '<ul>';
        $this->displayItems($menu, 1);
        echo '</ul>';
    }

    /**
     *   
     *
     * @param zmenu $items
     */
    private function displayItems ($items, $currentDepth)
    {
        $displayed = 0;
        foreach ($items as $item) {
            if ($this->getParam('number') > 0 && $displayed >= $this->getParam('number')) {
                break;
            }

            if ($item->nomenuitem || (false !== $this->getParam('menuid') && $item->menuid !== $this->getParam('menuid'))) {
                continue;
            }

            $itemClasses = array('page_item');
            //     
            if ($this->pagepath == $item->uri) {
                $itemClasses[] = 'current_page_item';
            }
            //      
            if (strpos($this->pagepath, $item->uri . '/') === 0) {
                $itemClasses[] = 'current_page_ancestor';
                //      
                if (false === strpos(substr($this->pagepath, strlen($item->uri) + 1), '/')) {
                    $itemClasses[] = 'current_page_parent';
                }
            }

            echo '<li class="' . implode(' ', $itemClasses) . '">';

            if (strpos($this->pagepath, $item->uri . '/') === 0 || strpos($item->uri, dirname($this->pagepath)) !== 0) {
                echo '<a href="' . $item->link . '"' . $this->nofollow . '>' . $item->name . '</a>';
            } else {
                echo '<a href="' . $item->link . '">' . $item->name . '</a>';
            }

            if ((!$this->getParam('depth') || $currentDepth < $this->getParam('depth')) && !$item->nosubmenu && $item->hasSubmenu() && $this->showSubmenu($item)) {
                echo '<ul>';
                $this->displayItems($item, $currentDepth + 1);
                echo '</ul>';
            }

            echo '</li>';
            ++$displayed;
        }
    }

    /**
     *  ,       
     *
     * @param zmenu $item  
     * @return boolean
     */
    private function showSubmenu ($item) {
        if ($this->getParam('type', 'minimal') == 'minimal') {
            if ($item->uri == $this->pagepath || strpos($this->pagepath, $item->uri . '/') === 0) {
                return true;
            } else {
                return false;
            }
        }

        return true;
    }

    /**
     *     
     *
     * @var string
     */
    private $nofollow;

    /**
     *    
     *
     * @var string
     */
    private $pagepath;

    /**
     *    
     *
     * @var array
     */
    protected $defaults = array(
        'type' => 'minimal',
        'depth' => 0,
        'menuid' => 'main',
        'childs_of' => '',
        'number' => 0,
    );
}

/**
 * @package    zengine
 * @subpackage widgets
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zwidget_tagcloud extends zwidget
{
    /**
     *   
     */
    public function displayContent () {
        $tags = zengine::zsource()->listUsedTags();
        if (empty($tags)) {
            return;
        }

        //        ,
        //     
        if ($this->getParam('number') > 0) {
            $tags = array_slice($tags, 0, $this->getParam('number'));
        }

        //     
        uasort($tags, array($this, 'tagcmp'));
        if ('DESC' == $this->getParam('order')) {
            $tags = array_reverse($tags, true);
        } elseif ('RAND' == $this->getParam('order')) {
            $keys = array_rand($tags, count($tags));
            foreach ($keys as $key) {
                $temp[$key] = $tags[$key];
            }
            $tags = $temp;
            unset($temp);
        }

        //      
        $counts = array(); {
        foreach ($tags as $key => $tag )
            $counts[$key] = $tag['count'];
        }
        $min_count = min($counts);
        $spread = max($counts) - $min_count;
        if ($spread <= 0) {
            $spread = 1;
        }
        $font_spread = $this->getParam('largest') - $this->getParam('smallest');
        if ($font_spread <= 0) {
            $font_spread = 1;
        }
        $font_step = $font_spread / $spread;

        //    
        $a = array();
        foreach ($tags as $link => $tag) {
            $count = $tag['count'];
            $tag_link = zengine::getModule('ztags')->getTagLink($link);
            $tag_id = isset($tag['id']) ? $tag['id'] : $link;
            $tag_name = $tag['name'];
            $a[] = "<a href='$tag_link' class='tag-link-$tag_id' title='" . $count . "' style='font-size: " .
                ($this->getParam('smallest') + (($count - $min_count) * $font_step))
                . $this->getParam('unit') . ";'>$tag_name</a>";
        }

        //   
        switch ($this->getParam('format')) {
        case 'list' :
            echo "<ul class='wp-tag-cloud'>\n\t<li>";
            echo implode("</li>\n\t<li>", $a);
            echo "</li>\n</ul>\n";
            break;
        default :
            echo implode("\n", $a);
            break;
        }
    }

    /**
     *    
     *
     * @param array $left
     * @param array $right
     * @return int
     */
    public function tagcmp ($left, $right) {
        $cmpkey = $this->getParam('orderby');
        if (!isset($left[$cmpkey]) || !isset($right[$cmpkey])) {
            $cmpkey = 'name';
        }

        $fnCmp = 'strcmp';
        if ($cmpkey == 'name') {
            $fnCmp = 'strnatcasecmp';
        }

        return $fnCmp($left[$cmpkey], $right[$cmpkey]);
    }

    /**
     *    
     *
     * @var array
     */
    protected $defaults = array(
        'smallest' => 8,
        'largest' => 22,
        'unit' => 'pt',
        'number' => 45,
        'format' => 'flat',
        'orderby' => 'name',
        'order' => 'ASC'
    );
}
