<?php
/**
 * Zebrum Lite
 *
 * 
 *
 *      ,    
 *    LICENSE.txt.
 *           URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: zcache.php 657 2009-04-28 06:55:29Z lks $
 */


/**
 *     
 *
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zcache
{
    /**
     *     TTL
     */
    const TTL_NEVER_EXPIRES = 0;

    /**
     *   
     *
     * @var string
     */
    private $_cachedir;

    /**
     *    
     *
     * @param string $cachedir    
     */
    public function __construct ($cachedir) {
    	if (!is_dir($cachedir)) {
    	    mkdir($cachedir, 0777);
    	}

        $this->_cachedir = realpath($cachedir);

    	if (!file_exists($this->_cachedir . '/drop.php')) {
    	    zutils::file_put_contents($this->_cachedir . '/drop.php', '<?php
foreach (new DirectoryIterator(dirname(__FILE__)) as $file) {
    if ($file->isDot()) {
        continue;
    }
    @unlink($file->getPathname());
}
echo "cache droped";
');
    	}
    }

    /**
     *       $name
     *
     * @param string $name
     * @param mixed $value
     */
    public function put ($name, $value) {
        if (0 == zengine::config()->get('engine.ttl')) {
            return;
        }
        $filename = $this->tempnam($this->_cachedir, 'tmp');
        if (null === $filename) {
            return;
        }
    	zutils::file_put_contents($filename, serialize($value));
    	if (file_exists($this->_cachedir . '/' . $name . '.cache')) {
    	   @unlink($this->_cachedir . '/' . $name . '.cache');
    	}
    	@rename($filename, $this->_cachedir . '/' . $name . '.cache');
    	zutils::chmod($this->_cachedir . '/' . $name . '.cache', 0666);
        zutils::touch($this->_cachedir . '/' . $name . '.cache', time());
    }

    /**
     *   tempnam,   ,    
     *    
     *
     * @param string $dir
     * @param string $prefix
     * @return string
     */
    public function tempnam ($dir, $prefix = '') {
    	if (!is_dir($dir)) {
    	    return null;
    	}

    	do {
    	   $filename = $dir . '/' . $prefix . mt_rand();
    	} while(file_exists($filename));

    	return $filename;
    }

    /**
     *       $name.   , 
     *  false
     *
     * @param string $name  
     * @param int $ttl      
     * @return mixed
     */
    public function get ($name, $ttl = -1) {
        if (isset($this->_cache[$name])) {
            return $this->_cache[$name];
        }
        if ($ttl == -1) {
            $ttl = zengine::config()->get('engine.ttl');
            if ($ttl == 0) {
                return false;
            }
        }
        $filename = $this->_cachedir . '/' . $name . '.cache';
        if (!file_exists($filename) || ($ttl && filemtime($filename) + $ttl < time())) {
            return false;
        }
    	$this->_cache[$name] = @unserialize(file_get_contents($filename));
    	return $this->_cache[$name];
    }

    /**
     *    
     *
     * @param string $name  
     * @return int
     */
    public function mtime ($name) {
        $filename = $this->_cachedir . '/' . $name . '.cache';
        if (file_exists($filename)) {
            return filemtime($filename);
        }

        return 0;
    }

    /**
     *  
     */
    public function erase () {
        foreach (new DirectoryIterator($this->_cachedir) as $file) {
            if ($file->isDot() || $file->isDir()) {
                continue;
            }
            @unlink($file->getPathname());
        }
    }

    /**
     *      
     *
     * @var array
     */
    private $_cache = array();
}
