<?php
/**
 * Zebrum Lite
 *
 * 
 *
 *      ,    
 *    LICENSE.txt.
 *           URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: zconfig.php 655 2009-04-28 05:22:28Z  $
 */


/**
 *       .    
 *    .
 *
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zconfig
{
    /**
     * .       ,  
     *   ini-,     
     *
     * @param array|string $config
     * @throws Exception
     */
    public function __construct ($config)
    {
        if (is_array($config)) {
            $this->_config = $config;
        } else {
            $configFile = 'config/' . $config;
            if (file_exists($configFile) && is_readable($configFile)) {
                $this->_config = parse_ini_file($configFile, true);
                $this->_hash = md5(serialize($this->_config));
            } else {
                throw new Exception('Unable to find config file');
            }
        }
    }

    /**
     *    .  
     * :
     *
     * <code>
     * $config = new zconfig('config.ini');
     * echo $config->section->property;
     * </code>
     *
     * @param string $key  
     */
    private function __get ($key)
    {
        if (isset($this->_config[$key])) {
            if (is_array($this->_config[$key])) {
                $this->_config[$key] = new zconfig($this->_config[$key]);
                return $this->_config[$key];
            }
            return $this->_config[$key];
        } else {
            return null;
        }
    }

    /**
     *    .  
     * :
     *
     * <code>
     * $config = new zconfig('config.ini');
     * echo $config->get('section.property');
     * </code>
     *
     * @param string $key  
     * @param mixed $defaultValue   
     * @return mixed
     */
    public function get ($key, $defaultValue = null)
    {
        if (isset($this->_config[$key])) {
            return $this->_config[$key];
        } else if (strpos($key, '.') !== false) {
            list ($_key, $key) = explode('.', $key, 2);
            if (isset($this->_config[$_key])) {
                return $this->$_key->get($key, $defaultValue);
            }
        }
        return $defaultValue;
    }

    /**
     *     ,     
     *
     * @param string $key
     * @param mixed $defaultValue
     */
    public function setDefault ($key, $defaultValue) {
    	if (null === $this->get($key)) {
    	    $this->_config[$key] = $defaultValue;
    	}
    }

    /**
     *   
     *
     * @param string $key
     * @param mixed $value
     */
    public function set ($key, $value) {
        $this->_config[$key] = $value;
    }

    /**
     *       
     *
     * @return array
     */
    public function toArray () {
    	return $this->_config;
    }

    /**
     *  - 
     *
     * @return string
     */
    public function getHash () {
    	return $this->_hash;
    }

    /**
     *  
     *
     * @var array
     */
    private $_config = array();

    /**
     * - 
     *
     * @var string
     */
    private $_hash;
}
