<?php
/**
 * Zebrum Lite
 *
 * 
 *
 *      ,    
 *    LICENSE.txt.
 *           URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: zengine.php 690 2009-05-21 03:02:47Z  $
 */

/** zconfig */
require_once 'zengine/classes/zconfig.php';

/** zcache */
require_once 'zengine/classes/zcache.php';

/** zsource */
require_once 'zengine/classes/zsource.php';

/** zpage */
require_once 'zengine/classes/zpage.php';

/** zrouter */
require_once 'zengine/classes/zrouter.php';

/** ztemplate */
require_once 'zengine/classes/ztemplate.php';

/** zutils */
require_once 'zengine/classes/zutils.php';

/** zwidget */
require_once 'zengine/classes/zwidget.php';

/** zsidebar */
require_once 'zengine/classes/zsidebar.php';

/** zsidebarmanager */
require_once 'zengine/classes/zsidebarmanager.php';

/**   */
require_once 'zengine/classes/widgets/defaults.php';

/**
 *    .     ,
 *      .
 *
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zengine
{
    /**
     *   
     *
     */
    const VERSION = '1.1.4';

    /**
     *   
     *
     * @return string
     */
    static public function getVersion () {
        return self::VERSION;
    }

    /**
     *      $uri
     *
     * @param string $uri  
     */
    static public function run ($uri)
    {
        self::init();

        //   
        if (file_exists('zengine/rewrite_rules.php') && is_readable('zengine/rewrite_rules.php')) {
            require_once 'zengine/rewrite_rules.php';
        }

        //       
        if (file_exists('zengine/init.php') && is_readable('zengine/init.php')) {
            require_once 'zengine/init.php';
        }

        //  
        try {
            self::$_zengine->dispatch($uri);
        } catch (Exception $e) {
            header('HTTP/1.1 500 Internal Server Error');
            echo $e->getMessage();
            // 500
        }
    }

    /**
     *   ,     
     */
    static public function init () {
        if (is_null(self::$_zengine) && !defined('ZENGINE_VERSION')) {
            define('ZENGINE_VERSION', self::getVersion());
        }

        self::$_zengine = new zengine();
        self::$_zengine->configure('config.ini');
        self::$_zengine->determineBaseUri();
        self::$_zengine->_zrouter = new zrouter();

        /**
         *   
         */
        date_default_timezone_set(zengine::config()->get('engine.timezone', 'Europe/Moscow'));

        /**
         *  ,    
         */
        if (! defined('ZENGINE_NOW')) {
            define('ZENGINE_NOW', date('Y-m-d H:i:s'));
        }

        /**
         *  ,  
         */
        $hash = self::$_zengine->_cache->get('confighash');
        if (false === $hash || $hash != self::$_zengine->getSettingsHash()) {
            self::$_zengine->_cache->erase();
            self::$_zengine->_cache->put('confighash', self::$_zengine->getSettingsHash());
        }
    }

    /**
     *   
     *
     * @return zrouter
     */
    public static function getRouter () {
    	return self::$_zengine->_zrouter;
    }

    /**
     *    
     *
     * @return zengine
     */
    static public function getInstance ()
    {
        return self::$_zengine;
    }

    /**
     *   uri
     *
     * @return string
     */
    public function getBaseUri ()
    {
        return $this->_baseuri;
    }

    /**
     *   URI  
     *
     */
    public function determineBaseUri () {
        if (defined('ZENGINE_BASEURI')) {
            $this->_baseuri = ZENGINE_BASEURI;
        } else {
            $baseuri = $this->config()->get('url.baseuri', '');

            /**
             *  url.baseuri     ,  
             *   
             */
            if (empty($baseuri)) {
                $baseuri = '/';

                /**
                 *        ,   
                 * 
                 */
                if ($_SERVER['REQUEST_URI'] != $this->_uri && substr($_SERVER['REQUEST_URI'], - strlen($this->_uri)) == $this->_uri) {
                    $baseuri = substr($_SERVER['REQUEST_URI'], 0, - strlen($this->_uri)) . '/';
                } else {
                    /**
                     *        
                     *  
                     */
                    $proposedDocumentRoot = rtrim(realpath(dirname(__FILE__) . '/../../'), DIRECTORY_SEPARATOR);
                    $realDocumentRoot = rtrim(realpath($_SERVER['DOCUMENT_ROOT']), DIRECTORY_SEPARATOR);
                    if ($proposedDocumentRoot != $realDocumentRoot && substr($proposedDocumentRoot, 0, strlen($realDocumentRoot)) == $realDocumentRoot) {
                        $baseuri = str_replace(DIRECTORY_SEPARATOR, '/', substr($proposedDocumentRoot, strlen($realDocumentRoot))) . '/';
                    }
                }
            } else {
                $baseuri = str_replace('//', '/', '/' . trim($baseuri, '/') . '/');
            }

            $this->_baseuri = $baseuri;
        }
    }

    /**
     *  -  
     *
     * @return string
     */
    public function getSettingsHash () {
    	return md5($this->getBaseUri(false) . $this->config()->getHash() . ZENGINE_VERSION);
    }

    /**
     *   .     ,   
     *
     * @param string $module
     * @return zmodule
     */
    static public function getModule ($module) {
        if (!isset(self::$_modules[$module])) {
            self::$_modules[$module] = null;
            $moduleFile = 'templates/' . zengine::config()->get('template.theme', 'default') . '/modules/' . $module . '/' . $module . '.php';
            if (! file_exists($moduleFile)) {
                $moduleFile = 'zengine/modules/' . $module . '/' . $module . '.php';
            }
            if (file_exists($moduleFile)) {
                include_once $moduleFile;
                $className = 'zmodule_' . $module;
                if (class_exists($className, false)) {
                    $zmodule = new $className();
                    if ($zmodule instanceof zmodule) {
                        self::$_modules[$module] = $zmodule;
                    }
                }
            }
        }
        return self::$_modules[$module];
    }

    /**
     *      
     *
     * @return string
     */
    public function getFilepath () {
    	return $this->_filepath;
    }

    /**
     *  true,    
     *
     * @return boolean
     */
    static public function debugMode () {
    	return self::$_zengine->_debug;
    }

    /**
     *   
     *
     * @return zconfig
     */
    static public function config ()
    {
        return self::$_zengine->_config;
    }

    /**
     *    
     *
     * @return zsource
     */
    static public function zsource ()
    {
        return self::$_zengine->_zsource;
    }

    /**
     *      
     *
     * @return zcache
     */
    static public function zcache ()
    {
        return self::$_zengine->_cache;
    }

    /**
     *   
     *
     * @return ztemplate
     */
    static public function ztemplate ()
    {
        return self::$_zengine->_ztemplate;
    }

    /**
     *  
     *
     * @param string $uri  
     */
    private function dispatch ($uri)
    {
        /**
         *   
         */
        $this->_uri = $uri;

        /**
         *    
         */
        $zpage = $this->_zrouter->route($uri);
        if (zengine::debugMode()) {
            ob_start();
            echo '<pre>';
            echo ' : ' . ZENGINE_NOW . "\n";
            if ($zpage) {
                echo '  : ' . $zpage->getProperty('publish') . "\n";
                echo ' : ' . $zpage->getTitle();
            } else {
                echo '  ';
            }
            echo '</pre>';
        }
        if (!$zpage || $zpage->getProperty('publish') > ZENGINE_NOW) {
            header('HTTP/1.1 404 Not Found');
            require_once 'zengine/classes/zpage/error.php';
            $zpage = zpage_error::getNotFound();
        }

        $this->_ztemplate = new ztemplate();
        echo $this->_ztemplate->drawup($zpage);
    }

    /**
     *  
     */
    private function __construct ()
    {
    }

    /**
     *        $uri 
     *   ,     
     *
     * @param string $uri  
     * @return int
     */
    public function extractSubpageNumber (&$uri) {
        $pageuripos = 2;
        $pagenumpos = 1;
        /**
         *     
         */
        $pattern = preg_quote($this->_config->get('split.url', '#pageuri#-#pagenum#'), '%');
        if (strpos($pattern, '#pageuri#') < strpos($pattern, '#pagenum#')) {
            $pageuripos = 1;
            $pagenumpos = 2;
        }

        $pattern = '%^' . str_replace(array('#pagenum#', '#pageuri#'), array('(\d+)', '(.+?)'), $pattern) . '$%';

        /**
         *      
         */
        $subpage = 1;
        if (preg_match($pattern, basename($uri), $matches)) {
            $uri = rtrim(str_replace(DIRECTORY_SEPARATOR, '/', dirname($uri)), '/') . '/' . $matches[$pageuripos];
            $subpage = $matches[$pagenumpos];
            if ($subpage == 1) {
                return 0;
            }
        }

        return $subpage;
    }

    /**
     *   
     *
     * @param string $configfile   ini-
     */
    private function configure ($configfile)
    {
        //    
        $this->_baseuri = null;

        $this->_config = new zconfig($configfile);
        $this->_debug = (bool) $this->_config->get('engine.debug', false);
        /**
         *        
         */
        $this->_config->setDefault('engine.charset', 'utf-8');
        $this->_config->setDefault('source.charset', 'utf-8');
        $this->_config->setDefault('source.extension', 'txt');
        $this->_config->setDefault('url.extension', 'html');
        /**
         *   
         */
        if (is_string($this->_config->get('directory.chmod'))) {
            $this->_config->set('directory.chmod', octdec($this->_config->get('directory.chmod')));
        } else {
            $this->_config->set('directory.chmod', 0777);
        }
        if (is_string($this->_config->get('file.chmod'))) {
            $this->_config->set('file.chmod', octdec($this->_config->get('file.chmod')));
        } else {
            $this->_config->set('file.chmod', 0666);
        }
        /**
         *    
         */
        $source = $this->_config->get('source.engine', 'txt');
        $sourceClass = 'zsource_' . $source;
        if (! class_exists($sourceClass, false)) {
            if (! file_exists('zengine/classes/zsource/' . $source . '.php')) {
                throw new Exception('Invalid zsource');
            }
            require_once 'zengine/classes/zsource/' . $source . '.php';
            if (! class_exists($sourceClass, false)) {
                throw new Exception('Class ' . $sourceClass . ' is not found');
            }
        }
        $this->_zsource = new $sourceClass();
        $this->_cache = new zcache('cache');
    }

    /**
     *    
     *
     * @var zconfig
     */
    private $_config;

    /**
     *     
     *
     * @var zsource
     */
    private $_zsource;

    /**
     * ,     
     *
     * @var string
     */
    private $_uri;

    /**
     *    ,     
     *
     * @var string
     */
    private $_filepath;

    /**
     *  uri
     *
     * @var string
     */
    private $_baseuri;

    /**
     *   
     *
     * @var zengine
     */
    static private $_zengine;

    /**
     *      
     *
     * @var zcache
     */
    private $_cache;

    /**
     *   
     *
     * @var array
     */
    static private $_modules;

    /**
     *   .      
     *
     * @var ztemplate
     */
    private $_ztemplate;

    /**
     *   
     *
     * @var zrouter
     */
    private $_zrouter;

    /**
     *   
     *
     * @var boolean
     */
    private $_debug;
}
