<?php
/**
 * Zebrum Lite
 *
 * 
 *
 *      ,    
 *    LICENSE.txt.
 *           URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.html
 * @version    $Id: zmenu.php 669 2009-04-28 18:04:20Z lks $
 */

/** zpagesorter */
require_once 'zengine/classes/zpagesorter.php';

/**
 *     .
 *
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zmenu implements Iterator, RecursiveIterator {
    /**
     *   
     *
     * @var string
     */
    public $name;

    /**
     *      
     *
     * @var string
     */
    public $uri;

    /**
     *  
     *
     * @var string
     */
    public $publish;

    /**
     *      
     *
     * @var string
     */
    public $link;

    /**
     *   
     *
     * @var array
     */
    public $tags;

    /**
     *    
     *
     * @var boolean
     */
    public $nomenuitem = false;

    /**
     *      
     *
     * @var boolean
     */
    public $nosubmenu = false;

    /**
     *  ,    
     *
     * @var int
     */
    public $menuid = 'main';

    /**
     *     
     *
     * @var array
     */
    public $subpages = array();

    /**
     *      
     *
     * @var string
     */
    private $menuorder = null;

    /**
     *   
     *
     * @param zpage $page
     * @return zmenu
     */
    public function appendPage (zpage $page) {
        $zmenu = new zmenu();
        $zmenu->parent = $this;
        if ($page->getProperty('nomenuitem')) {
            $zmenu->nomenuitem = true;
        }
        if ($page->getProperty('nosubmenu')) {
            $zmenu->nosubmenu = true;
        }
        $zmenu->menuorder = $page->getProperty('menuorder', null);
        $zmenu->menuid = $page->getProperty('menuid', $this->menuid);
        $zmenu->name = (string) $page->getMenu();
        $zmenu->uri = (string) $page->getPagepath();
        $zmenu->tags = $page->getTags();
        $zmenu->publish = (string) $page->getProperty('publish');
        $zmenu->setOrderProperties($page);
        $zmenu->link = $page->getLink();
        $zmenu->subpages = array(1 => $zmenu->link);
        for ($i = 2, $n = $page->countSubpages(); $i <= $n; ++$i) {
            $zmenu->subpages[$i] = $page->getSubpageLink($i);
        }
        $this->_items[] = $zmenu;
        return $zmenu;
    }

    /**
     *    ,      
     *
     * @param array $tags
     * @return array
     * @deprecated     zengine::zsource()->listUsedTags()
     */
    public function extractTags (&$tags) {
        $tags = zengine::zsource()->listUsedTags();
    }

    /**
     *  - ,    
     *
     * @return string
     */
    public function getHash () {
    	return $this->_hash;
    }

    /**
     *     
     *
     * @return zmenu
     */
    public function getParent () {
    	return $this->parent;
    }


    /**
     *     
     *
     * @return zpagesorter
     */
    public function getSorter () {
        if (empty($this->menuorder)) {
            $this->menuorder = strtolower(trim(zengine::config()->get('menu.order')));
            if (empty($this->menuorder)) {
                $this->menuorder = 'menu asc';
            }
        }
        if (!isset(self::$_sorter[$this->menuorder])) {
            self::$_sorter[$this->menuorder] = new zpagesorter($this->menuorder);
        }
        return self::$_sorter[$this->menuorder];
    }

    /**
     *     ,    
     *
     * @param string $pagepath
     * @return zmenu
     */
    public function find ($pagepath) {
    	if ($this->uri == $pagepath) {
    	    return $this;
    	}

    	foreach ($this as $submenu) {
    	    if ($submenu->find($pagepath)) {
    	        return $submenu->find($pagepath);
    	    }
    	}

    	return null;
    }

    /**
     *  - ,    
     *
     * @param string $hash
     */
    public function setHash ($hash) {
        $this->_hash = $hash;
    }

    /**
     *  true,     
     *
     * @return boolean
     */
    public function hasSubmenu () {
        foreach ($this->_items as $item) {
            //   
            if (! ($item->publish > ZENGINE_NOW)) {
                return true;
            }
        }
    	return false;
    }

    /**
     *  true,        
     *
     * @return boolean
     */
    public function hasChildren () {
    	return $this->current()->hasSubmenu();
    }

    /**
     *     ()
     *
     * @return zmenu
     */
    public function getChildren () {
    	return $this->current();
    }


    /**
     *     
     *
     * @return zmenu
     */
    public function current () {
    	return $this->_items[$this->_cursor];
    }

    /**
     *      
     */
    public function next () {
        do {
    	   ++$this->_cursor;
        } while (isset($this->_items[$this->_cursor]) && $this->_items[$this->_cursor]->publish > ZENGINE_NOW);
    }

    /**
     *  true,     
     */
    public function valid () {
    	return isset($this->_items[$this->_cursor]);
    }

    /**
     *      
     */
    public function key () {
    	return $this->_cursor;
    }

    /**
     *          
     */
    public function rewind () {
        //   
    	$this->_cursor = -1;
    	$this->next();
    }

    /**
     *     
     *
     */
    public function sort () {
        $this->getSorter()->sort($this->_items);
    	foreach ($this as $item) {
    	    $item->sort();
    	}
    }

    /**
     *    ,   
     *
     * @param zpage $page
     */
    private function setOrderProperties (zpage $page) {
        foreach ($this->getSorter()->getSortOrders() as $order => $by) {
            if (empty($this->$order)) {
                if (method_exists($page, 'get' . $order)) {
                    $this->$order = $page->{'get' . $order}();
                } else {
                    $this->$order = $page->getProperty($order);
                }
            }
        }
    }

    /**
     *    
     *
     * @param string $name  
     * @return mixed
     */
    public function getProperty ($name) {
        if (isset($this->$name)) {
    	   return $this->$name;
        }

        return null;
    }

    /**
     *   
     *
     * @var array
     */
    private $_items = array();

    /**
     *      
     *
     * @var int
     */
    private $_cursor = 0;

    /**
     *        
     *
     * @var zpagesorter
     */
    static private $_sorter = array();

    /**
     *     
     *
     * @var zmenu
     */
    private $parent = null;

    /**
     *  ,     
     *
     * @var string
     */
    private $_hash;
}
