<?php
/**
 * Zebrum Lite
 *
 * 
 *
 *      ,    
 *    LICENSE.txt.
 *           URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: zpage.php 664 2009-04-28 14:37:13Z lks $
 */

/** zmenu */
require_once 'zengine/classes/zmenu.php';

/** zmodule */
require_once 'zengine/classes/zmodule.php';

/** ztransform */
require_once 'zengine/classes/ztransform.php';

/**
 *  
 *
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zpage
{
    /**
     *   ,     
     *
     * @param array $properties    
     * @return zpage  
     */
    static public function createPage ($properties) {
    	$zpage = new zpage();
    	$zpage->_initProperties($properties);
    	return $zpage;
    }

    /**
     *    
     *
     * @param string $name  
     * @param string $default  -
     * @return string
     */
    public function getProperty ($name, $default = null) {
    	if (isset($this->properties[$name])) {
    	    return $this->properties[$name];
    	}
    	return $default;
    }

    /**
     *   
     *
     * @return array
     */
    public function getChildren () {
        if (!$this->getPagepath()) {
            return array();
        }
        return zengine::zsource()->getChildren($this->getPagepath());
    }

    /**
     *   
     *
     * @return string
     */
    public function getTitle () {
    	return $this->getProperty('title');
    }

    /**
     *      
     *
     * @return string
     */
    public function getPagepath () {
        return $this->getProperty('pagepath');
    }

    /**
     *   ,    
     *
     * @return array
     */
    public function getTags () {
        $tags = array();
        foreach (preg_split('%\s*,\s*%', trim($this->getProperty('tags')), null, PREG_SPLIT_NO_EMPTY) as $tag) {
            $tagutf8 = $tag;
            if (strcasecmp($this->properties['charset'], 'utf-8') !== 0) {
                $tagutf8 = iconv($this->properties['charset'], 'utf-8', $tagutf8);
            }
            $tags[zutils::filename($tagutf8)] = $tag;
        }
        return $tags;
    }

    /**
     *      /abs/path/to/index.html  
     * zengine::getBaseUri().
     *
     * @return string
     */
    public function getLink () {
    	return zengine::getInstance()->getBaseUri() . substr($this->getPagepath(), 1) . '.' . zengine::config()->get('url.extension');
    }


    /**
     *    
     *
     * @return string
     */
    public function getKeywords () {
    	return $this->getProperty('keywords');
    }

    /**
     *   
     *
     * @return string
     */
    public function getDescription () {
        return $this->getProperty('description');
    }

    /**
     *    (h1)
     *
     * @return string
     */
    public function getHeading () {
        $heading = $this->getProperty('h1');
        if (empty($heading)) {
            $heading = $this->getTitle();
        }
    	return $heading;
    }

    /**
     *    
     *
     * @return string
     */
    public function getMenu () {
        $menuitem = $this->getProperty('menu');
        if (empty($menuitem)) {
            $menuitem = $this->getHeading();
        }
        return $menuitem;
    }

    /**
     *   
     *
     * @return string
     */
    public function getBody () {
        $body = $this->getCache('htmlbody');
        if (false === $body) {
            if ($this->getProperty('nosplit') || !isset($this->_subpages[$this->_subpage])) {
                $body = $this->getProperty('body');
            } else {
                $body = $this->_subpages[$this->_subpage];
            }
            $filters = $this->getProperty('filter');
            if (empty($filters)) {
                $filters = zengine::config()->get('source.filter');
            }
            foreach (preg_split("%\s*,\s*%", $filters, null, PREG_SPLIT_NO_EMPTY) as $filter) {
                $filterClass = 'zfilter_' . $filter;
                if (!class_exists($filterClass, false)) {
                    if (file_exists('zengine/classes/zfilter/' . $filter . '.php')) {
                        require_once 'zengine/classes/zfilter/' . $filter . '.php';
                    }
                }
                if (class_exists($filterClass, false)) {
                    $filter = new $filterClass();
                    $body = $filter->filter($body);
                } else {
                    throw new Exception('Filter not found: ' . $filter);
                }
            }

            if ('off' != $this->getProperty('cache', 'on')) {
                $this->cacheIt('htmlbody', $body);
            }
        }

        return $body;
    }

    /**
     *   .     
     * 
     *
     * @param string $body
     */
    public function setBody ($rawBody) {
    	$this->setProperty('body', $rawBody);
    	$this->cacheIt('htmlbody', $rawBody);
    }

    /**
     *   
     *
     * @param string $name  
     * @param string $default  
     */
    public function setProperty ($name, $rawValue) {
        $this->properties[$name] = $rawValue;
    }

    /**
     *   ,     
     *
     * @return string
     */
    public function getTemplate () {
    	return $this->getProperty('template');
    }

    /**
     *        ,   .
     *
     * @param string $uri      
     * @param boolean $direct    
     * @return zpage
     */
    static public function find ($uri, $direct) {
    	/**
    	 *   ,   , 
    	 */
    	if (substr(basename($uri), 0, 1) == '.') {
    	    return null;
    	}

    	/**
    	 *    ,   ,   
    	 * 
    	 */
    	$page = self::_find($uri, $direct);
    	if (!$page) {
        	/**
        	 *     .     
        	 *  1.
        	 */
        	$subpage = zengine::getInstance()->extractSubpageNumber($uri);
        	if ($subpage == 0) {
        	    return null;
        	}

        	$page = self::_find($uri, $direct, $subpage);
    	}

    	return $page;
    }

    /**
     *         
     *  ,   .
     *
     * @param string $uri      
     * @param boolean $direct    
     * @param int $subpage  
     * @return zpage
     */
    static private function _find ($uri, $direct, $subpage = 1) {
        /**
         *      ,   ;
         *    ,  
         */
        if ($direct && zengine::zsource()->exists($uri)) {
            return self::load($uri, $subpage);
        } elseif (substr($uri, -1) == '/' && zengine::zsource()->exists($uri . 'index')) {
            return self::load($uri . 'index', $subpage);
        }

        return null;
    }

    /**
     *    
     *
     * @param int $subpage
     */
    public function setSubpage ($subpage) {
        $this->_subpage = $subpage;
    }

    /**
     *    
     *
     * @return int
     */
    public function getSubpage () {
    	return $this->_subpage;
    }

    /**
     *     
     *
     * @return int
     */
    public function countSubpages () {
    	return count($this->_subpages);
    }

    /**
     *         
     * ,   .  
     *
     */
    public function splitPage () {
    	$splitby = (int) zengine::config()->get('split.size', 0);
    	if ($splitby <= 0 ||
    	    (!zengine::config()->get('split.auto', false) && !$this->getProperty('split', 0)) ||
    	    (zengine::config()->get('split.auto', false) && $this->getProperty('nosplit', 0))) {
    	    $subpages = array(1 => $this->properties['body']);
    	    $this->_subpages = $subpages;
    	    return;
    	}

    	/**
    	 *    
    	 */
        $paragraphs = preg_split("/\n/", $this->properties['body']);

        /**
         *   
         */
        $subpage = 1;
        $subpages = array(1 => '');
        $subpages_len = array(1 => 0);
        foreach ($paragraphs as $paragraph) {
            if (isset($subpages[$subpage]) && !empty($paragraph) && $subpages_len[$subpage] > $splitby) {
                $subpage++;
            }
            if (!isset($subpages[$subpage])) {
                $subpages[$subpage] = '';
                $subpages_len[$subpage] = 0;
            }
            $strlen = 0;
            if (function_exists('mb_strlen')) {
                $strlen = mb_strlen($paragraph, zengine::config()->get('engine.charset'));
            } elseif (0 === strcasecmp(zengine::config()->get('engine.charset'), 'utf-8')) {
                if (@preg_match_all('%(\pL|.)%u', $paragraph, $matches)) {
                    $strlen = count($matches[1]);
                } else {
                    $strlen = strlen($paragraph);
                }
            } else {
                $strlen = strlen($paragraph);
            }
            $subpages_len[$subpage] += $strlen;
            $subpages[$subpage] .= $paragraph . "\n";
        }

        $this->_subpages = $subpages;
    }

    /**
     *    
     *
     * @param int $subpage
     */
    public function getSubpageLink ($subpage) {
        if ($subpage <= 1) {
            return $this->getLink();
        }
        $filepath = $this->getPagepath();
        $basepath = ltrim(str_replace(DIRECTORY_SEPARATOR, '/', dirname($filepath)) . '/', '/');
        $filename = str_replace(array('#pageuri#', '#pagenum#'), array(basename($filepath), $subpage), zengine::config()->get('split.url', '#pageuri#-#pagenum#'));
        $link = zengine::getInstance()->getBaseUri() . $basepath . $filename . '.' . zengine::config()->get('url.extension');
    	return $link;
    }

    /**
     *      
     *
     * @param string $pagepath   
     * @param int $subpage
     * @return zpage
     */
    static public function load ($pagepath, $subpage = null) {
        if (isset(self::$loadedPages[$pagepath])) {
            $zpage = self::$loadedPages[$pagepath];
            if (null !== $subpage) {
                $zpage->setSubpage($subpage);
                if ($zpage->countSubpages() < $subpage) {
                    return null;
                }
            }
            return $zpage;
        }

        if (zengine::zcache()->mtime(self::pagepathToCacheName($pagepath, $subpage))
            > zengine::zsource()->mtime($pagepath)) {
            $zpage = zengine::zcache()->get(self::pagepathToCacheName($pagepath, $subpage));
            if (false !== $zpage) {
                if (null !== $subpage) {
                    $zpage->setSubpage($subpage);
                    if ($zpage->countSubpages() < $subpage) {
                        return null;
                    }
                }
                return $zpage;
            }
        }

        if (null === $subpage) {
            $subpage = 1;
        }

        $zpage = new zpage();
        $zpage->_initProperties(zengine::zsource()->load($pagepath));
        $zpage->setSubpage($subpage);
        if ($zpage->countSubpages() < $subpage) {
            return null;
        }
        $zpage->properties['pagepath'] = $pagepath;
        if ($zpage->properties['pagepath'] == '/index') {
            $zpage->properties['publish'] = ZENGINE_NOW;
        }

        return self::$loadedPages[$pagepath] = $zpage;
    }

    /**
     * 
     *
     */
    public function __destruct () {
        if ($this->_updated && $this->_subpage <= $this->countSubpages()) {
    	    $this->_updated = false;
    	    zengine::zcache()->put(self::pagepathToCacheName($this->getPagepath(), $this->_subpage), $this);
    	}
    }

    /**
     *         
     *
     * @param string $pagepath
     * @return string
     */
    static private function pagepathToCacheName ($pagepath, $subpage = 1) {
    	$filename = 'page.' . str_replace('/', '.', substr($pagepath, 1));
    	if ($subpage > 1) {
    	    $filename .= '.sub-' . $subpage;
    	}
    	return $filename;
    }

    /**
     * 
     *
     */
    private function __construct ()
    {}

    /**
     *   
     *
     * @param array $properties
     */
    private function _initProperties ($properties) {
        $this->properties = $properties;
        if (!isset($this->properties['charset'])) {
            $this->properties['charset'] = zengine::config()->get('source.charset');
        }
        /**
         *       ,
         *   
         */
        $pageCharset = $this->properties['charset'];
        $htmlCharset = zengine::config()->get('engine.charset');
        if ($pageCharset != $htmlCharset) {
            //   
            $this->checkCharsets($pageCharset, $htmlCharset);
            foreach ($this->properties as &$value) {
                $value = @iconv($pageCharset, $htmlCharset, $value);
            }
            $this->properties['charset'] = $htmlCharset;
        }
        /**
         *    
         */
        $this->applyTransforms($this);
        /**
         *    
         */
        $this->splitPage();
    }

    /**
     *    .
     *
     * @param zpage $zpage  
     */
    private function applyTransforms (zpage $zpage) {
        $transforms = explode(',', trim(zengine::config()->get('source.transforms')));
        foreach ($transforms as $transform) {
            $transform = trim($transform);
            $transformClass = 'ztransform_' . $transform;
            if (!class_exists($transformClass, false)) {
                if (!file_exists('zengine/transforms/' . $transform . '/' . $transform . '.php')) {
                    //   
                    continue;
                }
                require_once 'zengine/transforms/' . $transform . '/' . $transform . '.php';
                if (!class_exists($transformClass, false)) {
                    //   
                    continue;
                }
            }
            $transformObject = new $transformClass();
            if (! $transformObject instanceof ztransform) {
                //   
                continue;
            }
            $transformObject->transform($zpage);
        }
    }

    /**
     *     
     *
     * @param string $pageCharset
     * @param string $charsetTo
     * @throws Exception
     */
    private function checkCharsets ($pageCharset, $htmlCharset) {
    	if (!@iconv($pageCharset, $pageCharset, 'example')) {
    	    throw new Exception('Exception (page ' . $this->properties['filepath'] . '): Invalid page charset "' . $pageCharset . '"');
    	}
        if (!@iconv($htmlCharset, $htmlCharset, 'example')) {
            throw new Exception('Exception (page ' . $this->properties['filepath'] . '): Invalid html charset ' . $htmlCharset);
        }
    }

    /**
     *     
     *
     * @var array
     */
    protected $properties = array();

    /**
     *     ,   
     *
     * @var unknown_type
     */
    private $_cache = array();

    /**
     *        
     *
     * @var boolean
     */
    private $_updated = true;

    /**
     *     
     *
     * @var array
     */
    private static $loadedPages = array();

    /**
     *  ,       
     *
     * @param string $name
     * @param mixed $value
     */
    public function cacheIt ($name, $value) {
    	$this->_cache[$name] = $value;
    	$this->_updated = true;
    }

    /**
     *     
     *
     * @param string $name
     * @return mixed
     */
    public function getCache ($name) {
    	if (isset($this->_cache[$name])) {
    	    return $this->_cache[$name];
    	}
    	return false;
    }

    /**
     *   
     *
     * @var int
     */
    private $_subpage = 0;

    /**
     *    
     *
     * @var array
     */
    private $_subpages = array();
}
