<?php
/**
 * Zebrum Lite
 *
 * 
 *
 *      ,    
 *    LICENSE.txt.
 *           URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: txt.php 686 2009-05-20 19:01:10Z  $
 */

/** zsource */
require_once 'zengine/classes/zsource.php';

/**
 *      txt
 *
 * @package    zengine
 * @subpackage zsource
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class zsource_txt implements zsource
{
    /**
     *     $pagepath     
     *
     * @param string $pagepath
     * @return boolean
     */
    public function delete ($pagepath)
    {
    	$filepath = self::$contentDir . '/' . $pagepath . '.' . zengine::config()->get('source.extension');
    	if (!@unlink($filepath)) {
    	    return false;
    	}
    	if (is_dir(self::$contentDir . '/' . $pagepath)) {
    	    @rmdir(self::$contentDir . '/' . $pagepath);
    	}
    	return true;
    }

    /**
     *       ,  
     *  content.     -   .
     *
     * @param string $parent    ,     
     * @return array
     */
    public function getAllPages ($parent = '') {
        $parent = '/' . trim(preg_replace("%[^a-zA-Z0-9-_/]+%", "", $parent), '/');

        if (!is_dir(self::$contentDir . $parent)) {
            return array();
        }

        $arPages = array();

        $extensionPattern = '%\.' . preg_quote(zengine::config()->get('source.extension'), '%') . '$%';
        $iterator = new RecursiveIteratorIterator(new RecursiveDirectoryIterator(self::$contentDir . $parent), RecursiveIteratorIterator::LEAVES_ONLY);
        foreach ($iterator as $file) {
            /* @var $file RecursiveDirectoryIterator */
            if (! $file->isFile() || substr($file->getFilename(), 0, 1) == '.' || strpos($iterator->getSubPathname(), '/.svn') !== false) {
                continue;
            }

            if (preg_match($extensionPattern, $file->getFilename())) {
                $pagepath = $this->filepath2pagepath($file->getPathname());
                $arPages[$pagepath] = $pagepath;
            }
        }

        return $arPages;
    }

    /**
     *    
     *
     * @param string $pagepath    
     * @return array
     */
    public function getChildren ($pagepath) {
        static $pregPattern = null;
        if (!$pregPattern) {
            $pregPattern = '%^((?!\.).*?)\.' . zengine::config()->get('source.extension') . '$%';
        }
        $arChildren = array();
        if (is_dir(self::$contentDir . $pagepath)) {
            $iterator = new DirectoryIterator(self::$contentDir . $pagepath);
            foreach ($iterator as $file) {
                /* @var $file DirectoryIterator */
                if (! preg_match($pregPattern, $file->getFilename())) {
                    continue;
                }
                $zpage = zpage::load($this->filepath2pagepath($file->getPathname()));
                if (! ($zpage->getProperty('publish') > ZENGINE_NOW)) {
                    $arChildren[] = $zpage;
                }
            }
        }
        return $arChildren;
    }


    /**
     *     
     *
     * @return zmenu
     */
    public function getMenu () {
        $menu = zengine::zcache()->get('menu', zcache::TTL_NEVER_EXPIRES);
        if (false !== $menu && $menu->getHash() == zengine::getInstance()->getSettingsHash()) {
            return $menu;
        }
        $menu = new zmenu();
        $menu->setHash(zengine::getInstance()->getSettingsHash());
        $this->appendMenu($menu, new DirectoryIterator(self::$contentDir));
        $menu->sort();
        zengine::zcache()->put('menu', $menu);
        return $menu;
    }

    /**
     *  true,   ,    - false
     *
     * @return boolean
     */
    public function exists ($pagepath) {
    	return file_exists(self::$contentDir . '/' . ltrim($pagepath, '/') . '.' . zengine::config()->get('source.extension'));
    }

    /**
     *    
     *
     * @param string $pagepath   
     * @return array
     */
    public function load ($pagepath) {
        return $this->loadFromFile(self::$contentDir . $pagepath . '.' . zengine::config()->get('source.extension'));
    }

    /**
     *      
     *
     * @param string $pagepath   
     * @return int
     */
    public function mtime ($pagepath) {
        if (file_exists(self::$contentDir . $pagepath . '.' . zengine::config()->get('source.extension'))) {
            return filemtime(self::$contentDir . $pagepath . '.' . zengine::config()->get('source.extension'));
        }
        return 0;
    }

    /**
     *    
     *
     * @param string $filename      
     * @return array
     */
    public function loadFromFile ($filename) {
        $properties = $this->parse(file_get_contents($filename));
        if (empty($properties['publish'])) {
            $properties['publish'] = date('Y-m-d H:i:s', filemtime($filename));
        }
        return $properties;
    }

    /**
     *    
     *
     * @param string $content
     * @return array
     */
    public function parse ($content) {
        $this->properties = array();
        $this->properties['body'] = preg_replace_callback("%^@@([^=]+)=([^\r\n]+)\r?\n%m", array($this, 'extractProperty'), $content);
        return $this->properties;
    }

    /**
     *    $pagepath.txt   .
     *
     * @param string $pagepath   
     * @param array $properties    
     * @return boolean
     *
     */
    public function save ($pagepath, array $properties) {
        $filepath = self::$contentDir . $pagepath . '.' . zengine::config()->get('source.extension');
        $content = array();
        if (isset($properties['body'])) {
            $body = $properties['body'];
            unset($properties['body']);
        }
        foreach ($properties as $property => $value) {
        	$content[] = '@@' . $property . '=' . $this->toSingleLine($value) . "\n";
        }
        $content[] = $body;

        if (!is_dir(dirname($filepath))) {
            mkdir(dirname($filepath), zengine::config()->get('directory.chmod'), true);
            if (self::$contentDir == dirname($filepath)) {
                file_put_contents(self::$contentDir . '/.htaccess', 'Deny from all');
            }
        } else {
            zutils::chmod(dirname($filepath), zengine::config()->get('directory.chmod'));
        }

        zutils::file_put_contents($filepath, implode($content));
        zutils::chmod($filepath, zengine::config()->get('file.chmod'));
        zutils::touch($filepath, time());
    	return true;
    }

    /**
     *    ,      ,
     *      ,   
     *
     *     :
     * <code>
     * array(
     *     array(
     *         'tag' => '- ',
     *         'uri' => 'kakoi-to-teg',
     *         'count' => 5
     *     )
     * );
     * </code>
     * :
     *     name -  
     *     uri -  ,       
     *     count -  ,   
     *
     * @return array
     */
    public function listUsedTags ()
    {
        $tags = zengine::zcache()->get('tags');
        if (false === $tags) {
            $tags = array();

            foreach (new RecursiveIteratorIterator($this->getMenu(), RecursiveIteratorIterator::SELF_FIRST) as $item) {
                /* @var $item zmenu */
                if ($item->publish > ZENGINE_NOW) {
                    continue;
                }
                foreach ($item->tags as $taguri => $tagName) {
                    if (isset($tags[$taguri])) {
                        ++$tags[$taguri]['count'];
                    } else {
                        $tags[$taguri] = array(
                            'count' => 1,
                            'name' => $tagName,
                            'uri' => $taguri
                        );
                    }
                }
            }

            uasort($tags, array($this, 'tagscmp'));
            zengine::zcache()->put('tags', $tags);
        }
        return $tags;
    }

    /**
     *        
     *
     * @param array $left
     * @param array $right
     * @return int
     */
    private function tagscmp ($left, $right)
    {
        if ($left['count'] < $right['count']) {
            return 1;
        } elseif ($right['count'] < $left['count']) {
            return -1;
        } else {
            return strcmp($left['name'], $right['name']);
        }
    }

    /**
     *        ,  
     *   $taguri
     *
     * @param string $taguri  
     * @return array
     */
    public function getPagesByTagUri ($taguri)
    {
        $pages = array();

        foreach (new RecursiveIteratorIterator($this->getMenu(), RecursiveIteratorIterator::SELF_FIRST) as $item) {
            /* @var $item zmenu */
            if ($item->publish > ZENGINE_NOW) {
                continue;
            }
            foreach ($item->tags as $uri => $name) {
                if ($taguri === $uri) {
                    $pages[] = $item->uri;
                    break;
                }
            }
        }

        return $pages;
    }

    /**
     *  
     *
     */
    public function beginTransaction ()
    { }

    /**
     *  ,    
     *
     */
    public function commit ()
    { }

    /**
     *  ,    
     *
     */
    public function rollBack ()
    { }

    /**
     *      
     *
     */
    public function update () {
        zengine::zcache()->erase();
    }

    /**
     *     $zmenu 
     *
     * @param zmenu $zmenu ,    
     * @param DirectoryIterator $directory
     */
    private function appendMenu ($zmenu, $directory) {
        static $pregPattern = null;
        if (!$pregPattern) {
            $pregPattern = '%^((?!\.).*?)\.' . zengine::config()->get('source.extension') . '$%';
        }
    	foreach ($directory as $file) {
    	    /* @var $file DirectoryIterator */
    	    if ($file->isFile() && preg_match($pregPattern, $file->getFilename(), $matches)) {
    	        $page = zpage::load($this->filepath2pagepath($file->getPathname()));
    	        if ($page) {
    	            if (!$page->getPagepath()) {
    	                //     
    	                return;
    	            }
    	            $submenu = $zmenu->appendPage($page);
    	            if (is_dir(self::$contentDir . $page->getPagepath())) {
    	                $this->appendMenu($submenu, new DirectoryIterator(self::$contentDir . $page->getPagepath()));
    	            }
    	        }
    	    }
    	}
    }

    /**
     *        
     *
     * @param string $filepath
     * @return string
     */
    private function filepath2pagepath ($filepath) {
        static $extLen = 0;
        if (!$extLen) {
            $extLen = -strlen(zengine::config()->get('source.extension')) - 1;
        }
    	return str_replace(DIRECTORY_SEPARATOR, '/', substr($filepath, strlen(self::$contentDir), $extLen));
    }


    /**
     *       load    
     *
     * @param array $matches
     * @return null
     */
    private function extractProperty ($matches) {
        $this->properties[strtolower($matches[1])] = $matches[2];
    	return null;
    }

    /**
     *      
     *
     * @param string $string
     * @return string
     */
    private function toSingleLine ($string) {
        return preg_replace("%\r?\n%", ' ', $string);
    }

    /**
     *       
     *
     * @var array
     */
    private $properties;

    /**
     * ,      
     *
     * @var string
     */
    static private $contentDir = 'content';
}
