<?php
/**
 * Zebrum Lite
 *
 * 
 *
 *      ,    
 *    LICENSE.txt.
 *           URL:
 * http://www.zebrum.ru/license.html
 *
 * @category   zebrum
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 * @version    $Id: ztemplate.php 655 2009-04-28 05:22:28Z  $
 */

/** ztemplate_exception*/
require_once 'zengine/classes/ztemplate/exception.php';

/** zhelper */
require_once 'zengine/classes/zhelper.php';

/**     */
require_once 'zengine/classes/ztemplate/functions.php';

/**
 *     
 *
 * @package    zengine
 * @copyright  Copyright (c) 2008 Zebrum.ru
 * @license    http://www.zebrum.ru/license.txt
 */
class ztemplate {
    /**
     *   
     *
     * @var string
     */
    private $theme;

    /**
     *  ,    
     *
     * @var string
     */
    private $template;

    /**
     *    
     *
     * @var zpage
     */
    public $zpage;

    /**
     *   html-
     *
     * @var string
     */
    public $charset;

    /**
     * .
     */
    public function __construct () {
        /**
         *    
         */
        $this->helper = new zhelper();
        $this->helper->ztemplate = $this;
    }

    /**
     *         
     *
     * @param zpage $zpage
     * @return string
     * @throws ztemplate_exception
     */
    public function drawup (zpage $zpage) {
        $this->zpage = $zpage;
        $this->helper->zpage = $zpage;
        /**
         *    
         */
        $theme = zengine::config()->get('template.theme', 'default');
        $template = $zpage->getTemplate();
        if (empty($template)) {
            $template = zengine::config()->get('template.default', 'template');
        }

        /**
         *    html-
         */
        $this->charset = zengine::config()->get('engine.charset');
        header('Content-Type: text/html; charset=' . $this->charset, true);

        /**
         *   ,   
         */
        if (file_exists('templates/' . $theme . '/functions.php')) {
            include_once 'templates/' . $theme . '/functions.php';
        }

        /**
         *  
         */
        try {
            $content = $this->applyTemplate($theme, $template);
        } catch (ztemplate_exception $e) {
            if ($theme == 'default') {
                throw $e;
            }
            $content = $this->applyTemplate('default', $template);
        }
        $content = $this->replaceSnippets($content);
        echo $content;
    }

    /**
     *    
     *
     * @return string
     */
    public function getTheme () {
        if (empty($this->theme)) {
            throw new ztemplate_exception('No theme used');
        }
    	return $this->theme;
    }

    /**
     *    
     *
     * @return string
     */
    public function getTemplate () {
        if (empty($this->template)) {
            throw new ztemplate_exception('No template used');
        }
    	return $this->template;
    }

    /**
     *      
     *
     * @param string $filename
     * @return string
     */
    public function loadTpl ($filename) {
    	//if (file_exists('templates/' . $this->theme ))
    }

    /**
     *    
     *
     * @param string $content
     * @return string
     */
    private function replaceSnippets ($content) {
    	$snippets = array(
    	   '~/' => zengine::getInstance()->getBaseUri(),
           '~zfull/' => 'http://'. $_SERVER['HTTP_HOST'] . zengine::getInstance()->getBaseUri(),
    	   '~ztpl/' => zengine::getInstance()->getBaseUri() . 'templates/' . $this->theme . '/',
    	);
    	return str_replace(array_keys($snippets), array_values($snippets), $content);
    }

    /**
     *  
     *
     * @param string $theme
     * @param string $template
     * @return string
     */
    private function applyTemplate ($theme, $template) {
        $templateFile = $template . '.' . zengine::config()->get('template.extension', 'php');
        if (!file_exists('templates/' . $theme . '/' . $templateFile) || !is_readable('templates/' . $theme . '/' . $templateFile)) {
            throw new ztemplate_exception("Unable to load template '{$theme}:{$template}'");
    	}

    	$this->theme = $theme;
    	$this->template = $template;

    	$templateVersion = $this->getTemplateVersion($theme, $template);
    	$this->checkBackwardCompatibility($templateVersion);

    	ob_start();
        include('templates/' . $this->theme . '/' . $templateFile);
        $content = ob_get_contents();
        ob_end_clean();
        return $content;
    }

    /**
     *    
     *
     * @param string $theme
     * @param string $template
     * @return string
     */
    private function getTemplateVersion ($theme, $template) {
    	if (!file_exists('templates/' . $theme . '/.version')) {
    	    //   ,      
    	    return '1.0.2';
    	}
    	return trim(file_get_contents('templates/' . $theme . '/.version'));
    }

    /**
     *       , 
     *   
     *
     * @param string $version
     * @throws Exception
     */
    private function checkBackwardCompatibility ($version) {
        if (version_compare(ZENGINE_VERSION, $version, '<')) {
            throw new Exception('           ' . $version);
        }
    }

    /**
     *  xhtml-     <head>
     *
     * @return string
     */
    public function head () {
        $htmls[] = '<title>' . $this->zpage->getTitle() . '</title>';
        $htmls[] = $this->headMeta('keywords', $this->zpage->getKeywords());
        $htmls[] = $this->headMeta('description', $this->zpage->getDescription());
        $htmls[] = '<meta http-equiv="Content-Type" content="text/html; charset=' . $this->charset . '" />';
    	return implode($htmls) . "\n";
    }

    /**
     *   <meta>
     *
     * @param string $name 
     * @param string $content 
     * @return string
     */
    private function headMeta ($name, $content) {
    	if (!empty($content)) {
    	    return '<meta name="' . htmlspecialchars($name) . '" content="' . htmlspecialchars($content) . '" />';
    	}
    	return '';
    }

    /**
     *   ,   ,   
     *
     * @param string $name  
     * @param array $arguments   
     * @return mixed
     */
    public function __call ($name, $arguments) {
        return call_user_func_array(array($this->helper, $name), $arguments);
    }

    /**
     *   
     *
     * @param zhelper
     */
    public $helper;
}
